%%-------------------------------------------------
%% generate a "block " finite difference matrix -- using fd3dB
%%-------------------------------------------------
Bx = [3 1 -1; 1 3 -1; 0.5 1 3];
Bz = 10.0*ones(3,3);
A = fd3dB(15,15,6,Bx,Bx,Bz) ;  
n = size(A,1);

spy(A) 
title([' block elliptic matrix of size  n = ', num2str(n)]) 
disp('pausing -- push any key to continue ') 
pause
close 
%%-------------------------------------------------
%% create artificial rhs
rhs = A * ([1:n]') ; 
%% random initial guess
sol0 = rand(n,1); 
%%    S S O R 
%%
% construct SSOR preconditioner 
 L = tril(A); D = diag(A); U = triu(A);
 SSOR.L = L; 
 SSOR.D = full(D);
 SSOR.U = U;
 SSOR.kiter = 10;
%--------------------------------------------------- 
% ready to call fgmres
 disp(' solution with SSOR prec') 
 [sol,res1,its] = fgmres(A, SSOR,'precLDU', rhs, sol0);
 disp(' ** done ** ') 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   ILU 
%%
%% construct I L U (0) preconditioner 
%%
 [L, U] = iluk(A,0) ;
 ILU = struct('L',L,'U',U) 
 [sol,res2,its] = fgmres(A, ILU,'precLU', rhs, sol0);

 disp(' ** done ** ') 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% construct I L U (1) preconditioner 
%%%
 [L, U] = iluk(A,1) ;
 ILU.L = L; ILU.U = U; 
 [sol,res3,its] = fgmres(A, ILU,'precLU', rhs, sol0);
 disp(' ** done ** ') 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%-------------------------------------------------- 
% call ILU factorization from matlab -- store result
% in struct PRE 
%
 disp(' solution with ILUT-GMRES') 
 opt.droptol = 0.001;
 opt.threshold = 0.0;
 [L, U] = luinc(A,opt);
 ILU.L = L; ILU.U = U;
 [sol,res4,its] = fgmres(A, ILU,'precLU', rhs, sol0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

semilogy(res1);

semilogy(res1,'m--','marker','+','LineWidth',2,'color','k')
     hold on

semilogy(res2,'g-.','marker','v','LineWidth',2,'color','g')

semilogy(res3,'b-','marker','d','LineWidth',2,'color','b')

semilogy(res4,'r-','marker','o','LineWidth',2,'color','b')
legend('SSOR(k)','ILU(0)','ILU(1)','ilut')
     hold off 
