%%-------------------------------------------------
%% generate a "block " finite difference matrix -- using fd3dB
%%-------------------------------------------------

A = create_testmatrix;
n = size(A,1);


spy(A) 
title([' block elliptic matrix of size  n = ', num2str(n)]);
pause
close 
%%
%%-------------------------------------------------
%% create artificial rhs
rhs = A * ([1:n]') ; 
%% random initial guess
sol = rand(n,1); 
%%-------------------------------------------------

%%-------------------------------------------------
% read parameters from file 
 ARMSopt = arms_params;
%%-------------------------------------------------- 
% call ILU factorization from matlab -- store result
% in struct PRE 
%
 disp(' solution with ILUT-GMRES') 

 [L, U,P] = luinc(A, ARMSopt.ilutolB);
 PRE = struct('L',L,'U',U,'P',P) ; 
%-------------------------------------------------- 
% call fgmres accelerator
 [sol,res1,its1] = fgmres(A, PRE,'precLU', rhs, sol,ARMSopt);
 disp(' ** done ** ') 
%--------------------------------------------------
% repeat with ILU(0) 

 disp(' solution with ILU(0)-GMRES') 
 rhs = A * ([1:n]') ; sol = rand(n,1); 
% 
 [L, U, P] = luinc(A,0) ; 
 PRE = struct('L',L,'U',U,'P',P) ;

[sol,res2,its2] = fgmres(A, PRE,'precLU', rhs, sol,ARMSopt);
 disp(' ** done ** ') 

%--------------------------------------------------
% repeat with ILU(2) 

 disp(' solution with ILU(2)-GMRES') 
 rhs = A * ([1:n]') ; sol = rand(n,1); 
% 
 [L, U,P] = iluk(A, 2); 
 PRE = struct('L',L,'U',U,'P',P) ;

 [sol,res3,its3] = fgmres(A, PRE,'precLU', rhs, sol,ARMSopt);
 disp(' ** done ** ') 
%--------------------------------------------------
% repeat with SSOR

 disp(' solution with SSOR(2)-GMRES') 
 rhs = A * ([1:n]') ; sol = rand(n,1); 
 L = tril(A); U = A - L + diag(A);
 PRE = struct('L',tril(A),'U',U,eye(n));

 [sol,res4,its4] = fgmres(A, PRE,'precLU', rhs, sol,ARMSopt) ; 
 disp(' ** done ** ') 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  plot convergence curves. 
 close

semilogy([0:its1],res1,'linestyle','--','marker','*','LineWidth',2,'color','r') 
 hold on;
semilogy([0:its2],res2,'linestyle','-.','marker','v','LineWidth',2,'color','b') 
semilogy([0:its3],res3,'linestyle','-','marker','d','LineWidth',2,'color','m')
semilogy([0:its4],res4,'linestyle','--','marker','+','LineWidth',2,'color','k')
legend('ILUT-gmres','ILU(0)-GMRES','ILU(2)-GMRES','SSOR(2)-GMRES') 


